//
//  FirmwareStorage.m
//  Bluzone
//
//  Created by Jason Clary on 7/28/16.
//  Copyright © 2016 Bluvision. All rights reserved.
//

#import "FirmwareStorage.h"

#import <Bluvision/Bluvision.h>
NSString * const BLUStoredConfigurationsArrayPath       = @"storedConfigurationsArray.plist";
NSString * const BLUStoredConfigurationIdentifierKey    = @"BLUStoredConfigurationIdentifier";
NSString * const BLUStoredConfigurationKey              = @"BLUStoredConfiguration";
NSString * const BLUStoredStagesArrayPath               = @"storedStagesArray.plist";
NSString * const BLUStoredStageIdentifierKey            = @"BLUStoredStageIdentifier";
NSString * const BLUStoredFirmwareStageTwoDescription   = @"BLUStoredFirmwareStageTwoDescription";
NSString * const BLUStoredStageKey                      = @"BLUStoredStage";

@implementation FirmwareStorage

+ (instancetype)defaultStorage {
    return [[FirmwareStorage alloc] init];
}

- (NSString *)storedStagesPath {
    return [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) firstObject] stringByAppendingPathComponent:BLUStoredStagesArrayPath];
}

- (NSArray *)storedStages {
    NSString *path = [self storedStagesPath];

    NSArray *storedStages = nil;
    @try {
        id storedStagesObject = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
        if ([storedStagesObject isKindOfClass:[NSArray class]]) {
            storedStages = storedStagesObject;
        }
    }
    @catch (NSException *exception) {
    }
    @finally {
    }

    return storedStages;
}

- (NSDictionary *)storedFirmwareInfoForBeacon:(BLUConfigurableBeacon *)beacon {
    NSArray *storedStages = [self storedStages];
    NSPredicate *identifierPredicate = [NSPredicate predicateWithFormat:@"(BLUStoredStageIdentifier == %@)", beacon.peripheralIdentifier.UUIDString];

    NSArray *results = [storedStages filteredArrayUsingPredicate:identifierPredicate];
    if (results.count > 0) {
        return [results firstObject];
    }

    return nil;
}

- (BLUBeaconFirmwareUpdateStage)storedStageForConfigurableBeacon:(BLUConfigurableBeacon *)beacon {
    NSDictionary *storage = [self storedFirmwareInfoForBeacon:beacon];

    BLUBeaconFirmwareUpdateStage stage = BLUBeaconFirmwareUpdateStageNone;

    stage = [storage[BLUStoredStageKey] unsignedIntegerValue];

    return stage;
}

- (BLUFirmwareImage *)storedFirmwareImageForConfigurableBeacon:(BLUConfigurableBeacon *)beacon {
    NSDictionary *storage = [self storedFirmwareInfoForBeacon:beacon];

    NSData *firmwareData = [NSData dataWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"firmware-cardv3" ofType:@"sfw"]];
    NSArray *images = [BLUFirmwareImage firmwareImagesForData:firmwareData configurableBeacon:beacon];

    for (BLUFirmwareImage *image in images) {
        if ([image.firmwareDescription isEqualToString:storage[BLUStoredFirmwareStageTwoDescription]]) {
            return image;
        }
    }

    return nil;
}

- (void)storeFirmwareUpdateStage:(BLUBeaconFirmwareUpdateStage)stage image:(BLUFirmwareImage *)image forConfigurableBeacon:(BLUConfigurableBeacon *)beacon {
    if (!beacon || !beacon.peripheralIdentifier) {
        return;
    }

    BLUBeaconFirmwareUpdateStage storedStage = [self storedStageForConfigurableBeacon:beacon];
    if (storedStage == stage) {
        return;
    }
    else if (storedStage != BLUBeaconFirmwareUpdateStageNone) {
        [self removeStoredStageForConfigurableBeacon:beacon];
    }

    NSArray *storedStages = [self storedStages];
    NSDictionary *stageToStore = @{BLUStoredStageIdentifierKey : beacon.peripheralIdentifier.UUIDString,
                                   BLUStoredStageKey           : @(stage),
                                   BLUStoredFirmwareStageTwoDescription : image.firmwareDescription};

    if (storedStages) {
        storedStages = [storedStages arrayByAddingObject:stageToStore];
    }
    else {
        storedStages = @[stageToStore];
    }

    [self saveStoredStages:storedStages];
}

- (void)saveStoredStages:(NSArray *)storedStages {
    NSString *path = [self storedStagesPath];
    if ([storedStages count]) {
        [NSKeyedArchiver archiveRootObject:storedStages toFile:path];
    }
    else {
        if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
            NSError *error;
            [[NSFileManager defaultManager] removeItemAtPath:path error:&error];
        }
    }
}

- (void)removeStoredStageForConfigurableBeacon:(BLUConfigurableBeacon *)beacon {
    if (!beacon || !beacon.peripheralIdentifier) {
        return;
    }

    NSMutableArray *storedStages = [[self storedStages] mutableCopy];

    if (storedStages) {
        NSPredicate *identifierPredicate = [NSPredicate predicateWithFormat:@"(BLUStoredStageIdentifier == %@)", beacon.peripheralIdentifier.UUIDString];
        NSArray *results = [storedStages filteredArrayUsingPredicate:identifierPredicate];
        [storedStages removeObjectsInArray:results];
        [self saveStoredStages:storedStages];
    }
}

- (NSString *)storedConfigurationsPath {
    return [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) firstObject] stringByAppendingPathComponent:BLUStoredConfigurationsArrayPath];
}

- (NSArray *)storedConfigurations {
    NSString *path = [self storedConfigurationsPath];

    NSArray *storedConfigurations = nil;
    @try {
        id storedConfigurationsObject = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
        if ([storedConfigurationsObject isKindOfClass:[NSArray class]]) {
            storedConfigurations = storedConfigurationsObject;
        }
    }
    @catch (NSException *exception) {
    }
    @finally {
    }

    return storedConfigurations;
}

- (BLUBeaconConfiguration *)storedConfigurationForConfigurableBeacon:(BLUConfigurableBeacon *)beacon {
    NSArray *storedConfigurations = [self storedConfigurations];
    NSPredicate *identifierPredicate = [NSPredicate predicateWithFormat:@"(BLUStoredConfigurationIdentifier == %@)", beacon.peripheralIdentifier.UUIDString];
    NSArray *results = [storedConfigurations filteredArrayUsingPredicate:identifierPredicate];
    NSDictionary *configurationDictionary = [results firstObject];
    BLUBeaconConfiguration *config = configurationDictionary[BLUStoredConfigurationKey];

    return config;
}

- (void)storeConfiguration:(BLUBeaconConfiguration *)configuration forConfigurableBeacon:(BLUConfigurableBeacon *)beacon {
    if (!configuration || !beacon.peripheralIdentifier) {
        return;
    }

    BLUBeaconConfiguration *config = [self storedConfigurationForConfigurableBeacon:beacon];
    if (config && [config isEqual:configuration]) {
        return;
    }
    else if (config) {
        [self removeStoredConfigurationForConfigurableBeacon:beacon];
    }

    NSArray *storedConfigurations = [self storedConfigurations];
    NSDictionary *storedConfiguration = @{BLUStoredConfigurationIdentifierKey : beacon.peripheralIdentifier.UUIDString,
                                          BLUStoredConfigurationKey           : configuration};

    if (storedConfigurations) {
        storedConfigurations = [storedConfigurations arrayByAddingObject:storedConfiguration];
    }
    else {
        storedConfigurations = @[storedConfiguration];
    }

    [self saveStoredConfigurations:storedConfigurations];
}

- (void)saveStoredConfigurations:(NSArray *)storedConfigurations {
    NSString *path = [self storedConfigurationsPath];
    if ([storedConfigurations count]) {
        [NSKeyedArchiver archiveRootObject:storedConfigurations toFile:path];
    }
    else {
        if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
            NSError *error;
            [[NSFileManager defaultManager] removeItemAtPath:path error:&error];
        }
    }
}

- (void)removeStoredConfigurationForConfigurableBeacon:(BLUConfigurableBeacon *)beacon {
    if (!beacon.peripheralIdentifier) {
        return;
    }

    NSMutableArray *storedConfigurations = [[self storedConfigurations] mutableCopy];

    if (storedConfigurations) {
        NSPredicate *identifierPredicate = [NSPredicate predicateWithFormat:@"(BLUStoredConfigurationIdentifier == %@)", beacon.peripheralIdentifier.UUIDString];
        NSArray *results = [storedConfigurations filteredArrayUsingPredicate:identifierPredicate];
        [storedConfigurations removeObjectsInArray:results];
        [self saveStoredConfigurations:storedConfigurations];
    }
}

- (void)clearStoredData {
    [self removeItemAtPath:[self storedStagesPath]];
    [self removeItemAtPath:[self storedConfigurationsPath]];
}

- (void)removeItemAtPath:(NSString *)path {
    if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
        NSError *error;
        [[NSFileManager defaultManager] removeItemAtPath:path error:&error];
    }
}


@end
